#!/bin/sh
#
# $Id: op5backup.sh,v 1.5 2006/06/21 07:49:58 jd Exp $
#
# Author: Peter Ostlin <peter@op5.se>
#
# Copyright(C) 2006 OP5 AB
# All rights reserved.
#
# This script is used to backup configuration, logfiles and any other 
# user specific files of a op5 Monitor/op5 Statistics/op5 Logserver 
# installation. The backup is sent to a backup-server using ftp or 
# copied to a local path (which could be a remote mount point).
# It can be executed manually from the command line or
# on a regular basis using cron.
#

echo "This program is obsolete. Please, use 'op5-backup' instead."
exit 0

# **  Configuration  **
# Config moved to /etc/op5backup.conf

# Source the config.
source /etc/op5backup.conf

# Start execution

# If any arg, print usage info
if [ $1 ]; then
  if [[ $1 != "-v" && ! $1 =~ "upgrade" ]]; then
	echo "This script is used to backup configuration and logfiles of a" 
	echo "OP5 Monitor/Statistics/Logserver installation and send these"
	echo "to a backup-server via ftp or store on a local path."
	echo "It can be executed manually from the command line or on a "
	echo "regular basis using cron."
	echo ""
	echo "Usage: /usr/sbin/op5backup.sh [ --upgrade ]"
	echo ""
	echo "With no arguments op5backup creates a full backup and place it locally"
	echo "or on a remote ftp server depending on configuration in /etc/op5backup.conf"
	echo "Add the '--upgrade' switch and op5backup create a limitied backup suitable"
	echo "for upgrades, for example when switching from i386 to x86_64 arch."
	echo ""
	echo "CONFIGURATION:"
	echo "Edit the settings in /etc/op5backup.conf to match your environment."
	echo "When upgrading an example configfile will be placed in "
	echo "/etc/op5backup.conf.rpmnew. Review it for additional config options."
	echo ""
	echo "USAGE:"
	echo "To manually take a backup simply execute /usr/sbin/op5backup.sh"
	echo "To setup cron to execute this script you need to edit the" 
	echo "crontab file. (Log on as root and execute 'crontab -e')"
	echo "For backups every monday at 01.59 enter the following:"
	echo " 59 01 * * 1 /usr/sbin/op5backup.sh  "
	echo "For backups at 01.59 the 1:st of every month:"
	echo " 59 01 1 * * /usr/sbin/op5backup.sh  "
	echo ""
	echo "To restore a backup, execute the op5restore.sh script with the"
	echo "backup-file as argument:"
	echo "#> /usr/sbin/op5restore.sh thebackup.tar.gz"
	echo ""
	exit 0
  fi
  verbosity=" -s"
  if [[ $1 =~ "upgrade" ]]; then
     upgrade_only="true"
  fi

fi


op5_upgrade_files="/etc/cron.* /etc/hosts /etc/pki/tls /etc/group* /etc/passwd* /etc/shadow* /etc/gshadow* /etc/nrpe.* "
op5_upgrade_files="$op5_upgrade_files /etc/op5backup.conf /etc/op5license /etc/nsca.cfg /etc/send_nsca.cfg"
op5_upgrade_files="$op5_upgrade_files /etc/samba /etc/postfix /etc/resolv.conf /etc/smsd.conf /etc/ssh "
op5_upgrade_files="$op5_upgrade_files /etc/yum.conf /etc/yum.repos.d /etc/yum /etc/sysconfig/network* "
op5_upgrade_files="$op5_upgrade_files /etc/sysconfig/clock /etc/sudoers /etc/rc.d "
#op5_files="$op5_only /etc/my.cnf /etc/ldap.conf /etc/lftp.conf"


# No args, do the backup
dir="$TMP/sys-backup"
datum=`date -I`
hostname=`hostname`
# Use custom backup-filename if set
if [ ! $backupfilename ]; then 
	backupfilename="$dir/$hostname-Backup"
	else
	backupfilename="$dir/$backupfilename"
fi

if [ $upgrade_only ]; then
  backupfilename="$dir/$hostname-Upgrade-Backup"
fi

# Only append date to filename if custom config var 'excludedate' is not set to yes
if [ "$excludedate" != "yes" ]; then
	backupfilename="$backupfilename-$datum"
fi
# backupfilename="$backupfilename.tar.gz"
if [ $upgrade_only ]; then
  echo ""
  echo "Running op5backup in upgrade backup mode."
fi

echo ""
echo "Transfer is set to: $transfer"
echo -n "Backup will be named: "
basename $backupfilename.tar.gz
if [ "$transfer" == "local" ]; then
    echo "and placed in the '$storagepath' directory"
elif [ "$transfer" == "ftp" ]; then
    echo "and placed in $backuppath on server $backupserver"
else
    echo "Transfer is set to '$transfer' which is an unsupported method."
fi


rm -rf $dir
mkdir -p -m 0700 $dir
mkdir -p -m 0700 $dir/opt
ret=$?
if [ $ret -ne 0 ]; then
    logger -p cron.warn "Failed to create tmp dir '$dir'" -t op5backup $verbosity
	exit 1
fi

mkdir -p -m 0700 $dir/opt/databases
mkdir -p -m 0700 $dir/opt/op5-versions

statistics_installed=`rpm -q statistics`
if [ $? -eq 0 ]; then
        echo "Backing up op5 Statistics data"
	mkdir -p -m 0700 $dir/opt/statistics
	mysqldump statistics > $dir/opt/databases/statistics-$datum.sql
	cp -a /opt/statistics/rra $dir/opt/statistics
	# Dump rrd data to xml to make arch switch possible
	rrddir="$dir/opt/statistics/rra"
	echo " - Dumping statistics rrd files. This might take a while"
	for f in $rrddir/*.rrd; do
#	    echo "rrdtool dump $f $f.xml"
	    rrdtool dump $f $f.xml
		bzip2 $f.xml
	done

	cp -a /opt/statistics/resource $dir/opt/statistics
	cp -a /opt/statistics/scripts $dir/opt/statistics
	if [ -d "/opt/statistics/plugins/weathermap/images" ]; then
	    mkdir -p $dir/opt/statistics/plugins/weathermap
	    cp -a /opt/statistics/plugins/weathermap/images $dir/opt/statistics/plugins/weathermap
	fi
	if [ -d "/opt/statistics/plugins/weathermap/configs" ]; then
	    mkdir -p $dir/opt/statistics/plugins/weathermap
	    cp -a /opt/statistics/plugins/weathermap/configs $dir/opt/statistics/plugins/weathermap
	fi
	# Save statistics version to file
	rpm -q --qf="%{version}\n" op5-statistics-release > $dir/opt/op5-versions/op5-statistics-version.txt
	echo "Statistics backup done"
fi
monitor_installed=`rpm -q monitor`
if [ $? -eq 0 ]; then
        echo "Backing up op5 Monitor data"
	mkdir -p -m 0700 $dir/opt/monitor
	mkdir -p -m 0700 $dir/opt/monitor/op5
	mkdir -p -m 0700 $dir/opt/monitor/op5/notify		
	mkdir -p -m 0700 $dir/opt/monitor/op5/networkmap	
	cp -a /opt/monitor/var $dir/opt/monitor
	cp -a /opt/monitor/etc $dir/opt/monitor
	if [ -d /opt/monitor/op5/nagiosgraph ]; then
	  mkdir -p -m 0700 $dir/opt/monitor/op5/nagiosgraph
	  cp -a /opt/monitor/op5/nagiosgraph/rrd $dir/opt/monitor/op5/nagiosgraph
	  # FIXME. Dump nagiosgraph data to xml
	  cp -a /opt/monitor/op5/nagiosgraph/map_custom $dir/opt/monitor/op5/nagiosgraph
	  cp -p /opt/monitor/op5/nagiosgraph/nagiosgraph.conf $dir/opt/monitor/op5/nagiosgraph/
	  cp -a /opt/monitor/op5/nagiosgraph/map* $dir/opt/monitor/op5/nagiosgraph/
	fi
	cp -a /opt/monitor/op5/notify/skins.mail $dir/opt/monitor/op5/notify
	cp -a /opt/monitor/op5/notify/skins.sms $dir/opt/monitor/op5/notify
	cp -a /opt/monitor/op5/networkmap/backgrounds $dir/opt/monitor/op5/networkmap
	# Dump rrd data to xml to make arch switch possible
	if [ -d /opt/monitor/op5/pnp ]; then
	  echo " - Dumping pnp rrd files. This might take a while"
	  mkdir -p -m 0700 $dir/opt/monitor/op5/pnp
	  cp -a /opt/monitor/op5/pnp/perfdata $dir/opt/monitor/op5/pnp/
	  pnpdir=$(ls $dir/opt/monitor/op5/pnp/perfdata)
	  for d in $pnpdir; do
	    if [ -d $dir/opt/monitor/op5/pnp/perfdata/$d ]; then
	        for f in $dir/opt/monitor/op5/pnp/perfdata/$d/*.rrd; do
			    rrdtool dump $f $f.xml
				bzip2 $f.xml
			done
		fi
	  done

	  cp -a /opt/monitor/op5/pnp/templates $dir/opt/monitor/op5/pnp/
	  cp -p /opt/monitor/op5/pnp/*.cfg $dir/opt/monitor/op5/pnp/
	fi
	mysqldump monitor_gui > $dir/opt/databases/monitor_gui-$datum.sql
	if [ -d /opt/monitor/op5/nacoma ]; then
	  mysqldump nacoma > $dir/opt/databases/nacoma-$datum.sql
	else
	  mysqldump monitor_webconfig > $dir/opt/databases/monitor_webconfig-$datum.sql
	fi
	if [ -d /var/lib/mysql/monitor_reports ]; then
	   mysqldump monitor_reports > $dir/opt/databases/monitor_reports-$datum.sql
	fi
	# SAve monitor version to file
	rpm -q --qf="%{version}\n" op5-monitor-release > $dir/opt/op5-versions/op5-monitor-version.txt
	echo "Monitor backup done"
fi	
if [ -d "/opt/plugins" ]; then
	mkdir -p -m 0700 $dir/opt/plugins
	# Bugfix line Carl 2007 september 17
	mkdir -p -m 0770 $dir/opt/plugins/webinject
	# End bugfix line
	cp -a /opt/plugins/custom $dir/opt/plugins/custom 
	cp -a /opt/plugins/webinject/testcases $dir/opt/plugins/webinject/testcases
fi	
logserver_installed=`rpm -q op5-logserver`
if [ $? -eq 0 ]; then
	mkdir -p -m 0700 $dir/opt/logserver
	if [ $(rpm -q --queryformat="%{VERSION}\n"  `rpm -qa | grep logserver-gui` | cut -d '.' -f 1) -eq 2 ]
	then
	    mysqldump logs filterrules filters globalsettings users usersettings reports db_version > $dir/opt/databases/logs-$datum.sql
	    mysqldump --no-data logs messages >> $dir/opt/databases/logs-$datum.sql
	else
	    pg_dump -U op5lsu -f $dir/opt/databases/logs-$datum.sql -C -T 'messages_[0-9]*' op5lsu
	fi
	# Save logserver version to file
	rpm -q --qf="%{version}\n" op5-logserver > $dir/op5-versions/op5-logserver-version.txt
fi	

# Backup dokuwiki if it exists
if [ -d "/var/www/html/dokuwiki" ]; then
        mkdir -p -m 0770 $dir/var/www/html/dokuwiki
	mkdir -p -m 0770 $dir/var/www/html/dokuwiki/data
	mkdir -p -m 0770 $dir/var/www/html/dokuwiki/lib
	cp -a /var/www/html/dokuwiki/data/pages $dir/var/www/html/dokuwiki/data
	cp -a /var/www/html/dokuwiki/data/cache $dir/var/www/html/dokuwiki/data
	cp -a /var/www/html/dokuwiki/data/attic $dir/var/www/html/dokuwiki/data
	cp -a /var/www/html/dokuwiki/data/media $dir/var/www/html/dokuwiki/data
	cp -a /var/www/html/dokuwiki/lib/tpl $dir/var/www/html/dokuwiki/lib
	cp -a /var/www/html/dokuwiki/conf $dir/var/www/html/dokuwiki
fi

# Backup LDAP if it exists
if [ -e "/opt/op5sys/etc/ldapsecret" ]; then
	test -d $dir/etc/ || mkdir -p -m 0755 $dir/etc
	test -d $dir/tmp/ || mkdir -p $dir/tmp/
	cp -a /etc/openldap $dir/etc/openldap
	mkdir -p $dir/etc/httpd/conf.d/
	if [ -f /etc/httpd/conf.d/op5ldapauth.conf ]; then
	    cp -a /etc/httpd/conf.d/op5ldapauth.conf $dir/etc/httpd/conf.d/
	fi
	/usr/sbin/slapcat > $dir/tmp/slapcat.ldif
fi

# Backup op5sys if it exists
if [ -d "/opt/op5sys" ]; then
	test -d $dir/opt/ || mkdir -p $dir/opt/
	cp -a /opt/op5sys $dir/opt/op5sys
fi

# Backup user paths (from the userdir and userfile arrays in monback.conf)
for i in ${userdir[*]}; do 
	if [ -d $i ]; then
		mkdir -p -m 0770 $dir$i
		cp -a --parents $i $dir
	fi
done
for i in ${userfile[*]}; do 
	if [ -f $i ]; then
		newdir=`dirname $dir$i`
		mkdir -p -m 0770 $newdir
		cp -p $i $newdir
	fi
done



# Create the tar-ball
cd $dir
backupfilename="$backupfilename.tar"
tar cf $backupfilename * 
cd /
if [ $upgrade_only ]; then
  tar rf $backupfilename $op5_upgrade_files 2>&1 | grep -v "tar: Removing leading"
else
  tar rf $backupfilename etc
fi
gzip -S .gz  $backupfilename
backupfilename="$backupfilename.gz"

#tar cvzf $dir/Monitor-Backup-$datum.tar.gz *
ret=$?
if [ $ret -ne 0 ]; then
    logger -p cron.warn "Backup tarfile creation failed" -t op5backup $verbosity
	exit 1
fi

# Determin what to do with backupfile.
if [ "$transfer" == "ftp" ]; then
# Send to ftp server.
#	lftp -u $backupuser,$backuppass $backupserver/$backuppath -e \
#	"mkdir $datum; cd $datum && put $dir/Monitor-Backup-$datum.tar.gz; quit"

        lftp -e "set net:max-retries 3 && open $backupserver/$backuppath -u $backupuser,$backuppass || quit 1; \
	(mkdir $datum && cd $datum && put $backupfilename) || quit 1; quit 0"
	ret=$?
	if [ $ret -ne 0 ]; then
	    logger -p cron.warn "Backup upload failed" -t op5backup $verbosity
		exit 1
	fi
else if [ "$transfer" == "local" ]; then
       if [ -d $storagepath ]; then
	    cp $backupfilename $storagepath
	else 
	    logger -p cron.warn "Backup creation or upload failed, path '$storagepath' not found" -t op5backup $verbosity
		exit 1
	fi
	else
	logger -p cron.warn "Backup creation or upload failed, error in /etc/op5backup.conf" -t op5backup $verbosity
	exit 1
  fi
fi


ret=$?
if [ $ret -eq 0 ]; then
	logger -p cron.notice "Backup successfully completed" -t op5backup $verbosity
else 
	logger -p cron.warn "Backup creation or upload failed" -t op5backup $verbosity
fi

rm -rf $dir

exit $ret
